
# q-expansion of newform 318.2.a.e, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 318
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-1], [0], [1], [5], [0], [2], [-1], [3], [-1], [-4], [0], [-9], [0], [6], [-1], [-4], [-7], [1], [7], [-14], [-8], [8], [-12], [13], [12], [-4], [12], [-6], [-4], [8], [1], [-3], [-5], [18], [2], [11], [-8], [-20], [-16], [0], [-6], [0], [24], [7], [17], [10], [1], [27], [20], [-29], [-11], [5], [-8], [-22], [-3], [-6], [-29], [23], [-16], [7], [-21], [-26], [0], [16], [17], [-34], [6], [24], [-22], [15], [-8], [24], [25], [-27], [1], [34], [7], [-10], [11], [22], [19], [32], [-17], [40], [-2], [8], [0], [15], [34], [33], [41], [-8], [-16], [-20], [-21], [18], [18], [16], [40], [2], [-10], [2], [11], [20], [-7], [-3], [-20], [14], [0], [-32], [22], [41], [4], [8], [27], [-40], [22], [34], [-42], [0], [45], [-4], [-1], [21], [-36], [-45], [-25], [-9], [24], [-29], [-18], [-43], [6], [33], [-44], [-6], [0], [20], [-15], [-8], [-24], [37], [48], [-3], [24], [45], [-18], [26], [-56], [-34], [-58], [15], [-1], [-30], [32], [6], [6], [-23], [27], [41], [-12], [47], [3], [-45], [16], [51], [-52]]
